"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.decode = void 0;

var _remixLib = require("remix-lib");

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/* BEWARE: The issue ignoring logic relies on the structure of these regular
 * expressions, namely it is important a single capturing group catches all
 * SWC IDs in the line regex, and that there are two capturing groups in
 * the block regex, where the first group captures "dis", or "en" piece of
 * the command, and the second one captures SWC IDs. */
var REGEX_IGNORE_LINE = /\/\/ mythx-disable-line(\s?swc-\d+,?\s*)*$/;
var REGEX_IGNORE_BLOCK = /\/\* mythx-(dis|en)able(\s?swc-\d+,?\s*)*\s*\*\//g;

var getDecodedIssues = function getDecodedIssues(_ref) {
  var issuesData = _ref.issuesData,
      sourceCodeFiles = _ref.sourceCodeFiles,
      decoder = _ref.decoder,
      rootSourceList = _ref.rootSourceList;
  return issuesData.issues.map(function (issue) {
    if (!issue.locations) {
      return issue;
    }

    var decodedLocations = issue.locations.map(function (location) {
      // We need the source map to decode
      if (!location.sourceMap) {
        return [];
      } // We can't decode bytecode locations, those should be ignored
      // It returns an empty array because the object returned by this function needs to
      // be iterable


      if (location.sourceType === 'raw-bytecode') {
        return [];
      }

      if (location.sourceMap) {
        var ary = location.sourceMap.split(':');
        var sourceLocation = {
          length: parseInt(ary[1], 10),
          start: parseInt(ary[0], 10),
          fileIndex: parseInt(ary[2], 10)
        };
        var lineBreakPositions;

        try {
          var sourceList;

          if (Array.isArray(location.sourceList) && location.sourceList.length > 0) {
            // we check if the issue contains a correct sourceList
            sourceList = location.sourceList;
          } else if (Array.isArray(issuesData.sourceList) && issuesData.sourceList.length > 0) {
            // if not we use the sourceList on the top level issues api response
            sourceList = issuesData.sourceList;
          } else {
            // as a last resort we use the sourceFileNames from the original API input request
            sourceList = rootSourceList;
          }

          var sourceFileName;
          /**
           * Ideally this should be handled on the sourceList assignement, but I'm
           * making it this way in order to avoid side effects. We make a check to see if the source
           * file name is a 65 bit hex string and fallback to the alternative way of getting the source file name.
           */

          if (sourceList[sourceLocation.fileIndex].length === 66 && sourceList[sourceLocation.fileIndex].indexOf('0x') === 0) {
            sourceFileName = location.sourceList[sourceLocation.fileIndex];
          } else {
            sourceFileName = sourceList[sourceLocation.fileIndex];
          }

          var sourceCode = getSourceCodeFromFileName(sourceFileName, sourceCodeFiles);
          lineBreakPositions = decoder.getLinebreakPositions(sourceCode);
          var loc = decoder.convertOffsetToLineColumn(sourceLocation, lineBreakPositions);
          var ignore = false;
          var swcID = issue.swcID.toLowerCase();
          /* Checks for ignore line command */

          var line = loc.start.line;
          line = sourceCode.slice(line ? 1 + lineBreakPositions[line - 1] : 0, lineBreakPositions[line]).toLowerCase();
          var match = line.match(REGEX_IGNORE_LINE);
          /* Here `match[1]` is `undefined` when the SWC capture group of regex
           * has not matched anything, which means that regex matches just
           * `// mythx-ignore-line`, thus all issues on this line should be
           * ignored. Otherwise, when the comment includes actual SWC IDs
           * to ignore, `match[1]` equals the last value captured in the
           * line by the group, thus we need to search whether this issue SWC
           * is present in the entire matching subsrting (match[0] element). */

          if (match) {
            ignore = !match[1] || match[0].includes(swcID);
          }

          if (!ignore) {
            /* If the line is not ignored, we need to check for `mythx-disable`,
             * and `mythx-enable` comments in the source code prior the issue.
             */
            var code = sourceCode.slice(0, sourceLocation.start).toLowerCase();
            var matches = code.match(REGEX_IGNORE_BLOCK);

            if (matches) {
              matches.forEach(function (m) {
                if (m.startsWith('/* mythx-disable')) {
                  if (!m.includes('swc') || m.includes(swcID)) ignore = true;
                } else {
                  if (!m.includes('swc') || m.includes(swcID)) ignore = false;
                }
              });
            }
          }

          if (loc.start) {
            // Adjust because routines starts lines at 0 rather than 1.
            loc.start.line++;
          }

          if (loc.end) {
            loc.end.line++;
          }

          return [loc.start, loc.end, ignore];
        } catch (error) {
          console.debug(error);
          return [];
        }
      }
    });
    var numberOfDecodedLocations = decodedLocations.reduce(function (accumulator, currentValue) {
      return accumulator + currentValue.length;
    }, 0);

    if (decodedLocations && numberOfDecodedLocations > 0) {
      // return a decoded locations object if we can decode locations
      return _objectSpread({}, issue, {
        decodedLocations: decodedLocations
      });
    } else {
      // if we can not decode the locations, make the decodedLocations object null
      return _objectSpread({}, issue, {
        decodedLocations: null
      });
    }
  });
};

var getSourceCodeFromFileName = function getSourceCodeFromFileName(sourceFileName, sourceFiles) {
  if (sourceFiles[sourceFileName]) {
    return sourceFiles[sourceFileName].source;
  }

  return undefined;
};

var decode = function decode(apiIssuesResponse, apiRequest) {
  try {
    var sourceFileNames = Object.keys(apiRequest.sources);
    var decoder = new _remixLib.SourceMappingDecoder();

    if (apiRequest.sources && sourceFileNames.length > 0) {
      var decoded = apiIssuesResponse.map(function (item) {
        var decodedIssues = getDecodedIssues({
          issuesData: item,
          sourceCodeFiles: apiRequest.sources,
          rootSourceList: sourceFileNames,
          decoder: decoder
        });
        return _objectSpread({}, item, {
          issues: decodedIssues
        });
      });
      return decoded;
    }

    console.warn("Decoder: No source file names provided");
    return apiIssuesResponse;
  } catch (error) {
    console.error("Decoder: Invalid input");
    return apiIssuesResponse;
  }
};

exports.decode = decode;